use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
BundleAuthordep
CheckFor
Contributors
CorrectBranch
DROLSKY
DROLSKY's
Dave
DevTools
Dist
Etheridge
Fowler
Git
Karen
License
MakeMaker
Mark
MaybeFileWriter
PayPal
PerlLinterConfigFiles
Plugin
PluginBundle
Pod
Precious
Role
Rolsky
Rolsky's
Test
Weaver
WeaverConfig
Zilla
autarch
drolsky
ether
ini
lib
mailmap
mark
optimizations
perlcriticrc
perltidyrc
toml
